package fi.jyu.mit.ohj2;

import java.io.*;

/**
 * Readkey.java
 *
 * Luokka yhden merkin lukemiseen,
 * tarvitsee polkuun joko tiedoston
 * libreadkey.so tai readkey.dll,
 * muutoin vaatii enterin painalluksen
 *
 * @author Markku Vire, Jukka Mäntylä
 * @version 0.96 12.2.2003
 */
class Readkey
{
  private static native char getCharNative();
  private static boolean islib = false;
  private static BufferedReader linereader;

  static {
    try {
      System.loadLibrary("readkey");
      islib = true;
    }
    // Jos ei pystytä lataamaan kirjastoa, niin
    // käytetään default-versiota merkin lukemisessa
    catch (UnsatisfiedLinkError e) { e.getMessage(); }
    catch (SecurityException e)    { e.getMessage(); }

    init();

  }

  
  /**
   * Alustetaan lukijaluokka
   */
  public static void init() {
      if (!islib) linereader = new BufferedReader(new InputStreamReader(System.in));
  }
  
  /**
   *
   * Lukee yhden merkin näppäimistöltä
   * @return Palauttaa luetun merkin
   */
  public static char getChar() {
    if (islib) return getCharNative();
    // default-versio:
    String s = "";
    try {
      s = linereader.readLine();
    } catch (IOException ex) { 
        //
    }
    if ( ( s == null ) || ( s.equals("") ) ) return '\n';
    return s.charAt(0);
  }

  /**
   * Palauttaa tiedon siitä onko käyttöjärjestelmäkohtainen
   * readkey-kirjasto käytössä
   * @return Palauttaa tiedon onko readkey-kirjasto käytössä
   */
  public static boolean isLibLoaded() {
    return islib;
  }

  /** 
   * Testataan Readkey-kirjastoa
   * @param args ei käytössä
   */
  public static void main(String[] args)
  {
    if (isLibLoaded()) 
      System.out.print("readkey-kirjasto ladattu.\nAnna merkki:");
    else 
      System.out.print("readkey-kirjastoa ei löytynyt.\nAnna kirjain ja paina enter:");
    char c = Readkey.getChar();
    if (isLibLoaded()) System.out.println(c);
    System.out.println("Annoit merkin: " + c);
  }

}
