package fi.jyu.mit.ohj2;
import java.io.*;

/**
 * <p>Title: Syotto</p>
 * <p>Description: Aliohjelmia tietojen lukemiseen päätteeltä</p>
 * <p>Lukeminen on vaihdettu käyttämään omaa RiviLukijaa jotta
 * syötön uudelleensuuntaus toimisi paremmin</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: jyu </p>
 * @author Vesa Lappalainen
 * @version 1.0/08.01.2003
 * @version 1.1/25.01.2003
 * @version 1.2/11.03.2007
 */

public class Syotto {

  /**
   * Lukija joka lukee yden rivin InputStreamista
   * @author vesal
   * @version 11.3.2007
   */
  public static class RiviLukija {
    /**
     * Jos edellinen merkki on CR, niin seuraava LF unohdetaan
     */
    //private boolean lastCR = false;
    //private InputStream in;
    private final BufferedReader inb; 
    
    /**
     * Alustetaan lukija lukemaan tiettyä tietovirtaa
     * @param in tietovirta jota luetaan
     */
    public RiviLukija(InputStream in) {
      //this.in = in;
      inb = new BufferedReader(new InputStreamReader(in));
    }
    
    /**
     * Luetaan rivi tietovirrasta. rivinvaihtona kelpaa
     * cr, lf  tai crlf
     * @return luettu rivi tai null jos tietovirta "loppu"
     */
    public String lueRivi() {
    /*  
      StringBuffer sb = new StringBuffer();
      int n = 0;
      while ( true ) {
        int c;
        try {
          c = in.read();
          if ( c < 0 ) if ( n == 0 ) return null; else break;
          n++;
          if ( c == '\r' ) { lastCR=true; break; }
          if ( c == '\n' ) if ( lastCR ) { lastCR = false; continue; } else break;
          lastCR = false;
        } catch (IOException e) {
          break;
        }
        sb.append((char)c);
      }
      return sb.toString();
    */  
      
      try {
        return inb.readLine();
      } catch (IOException e) {
        return "";
      }
    }
    
  }
  
  /**
   * Tietovirran lukija
   */
  private static RiviLukija lukija = new RiviLukija(System.in);

  /**
   * Alustaa sisäisen lukijan uudelleen mahdollisen suuntauksen jälkeen.
   *
   */
  public static void alusta() {
      lukija = new RiviLukija(System.in);    
  }
  
  
  /**
   * Luetaan seuraava rivi System.in-tieotovirrasta
   * @return luettu rivi
   */
  public static String lueRivi() {
    return lukija.lueRivi();
  }
  
  
  /**
   * Aliohjelmalla kysytään käyttäjältä arvo merkkijonolle.
   * Jos tulOletus on tosi, tulostetaan oletusarvo.
   * Jos käyttäjä painaa pelkästään ret, palautetaan oletusarvo
   *
   * @param kysymys käyttäjälle tulostettava kysymys
   * @param oletus arvo jota käytetään jos painetaan pelkkä ret
   * @param tulOletus tulostetaanko oletus käyttäjälle
   * @return käyttäjän syöttämä merkkijono
   */
  public static String kysy(String kysymys, String oletus, boolean tulOletus) {
    //BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
    while ( true ) {
      System.out.print(kysymys);
      if ( tulOletus ) System.out.print(" (" + oletus + ")");
      System.out.print(" >");
      String s = "";
      //try {
        //s = in.readLine();
        s = lueRivi();
      //} catch (IOException ex) {
      //  continue;
      //}
      if ( ( s == null ) || ( s.equals("") ) ) return oletus;
      return s;
    }

  }

  /**
   * Aliohjelmalla kysytään käyttäjältä arvo merkkijonolle.
   * Tulostetaan oletusarvo.
   * Jos käyttäjä painaa pelkästään ret, palautetaan oletusarvo
   *
   * @param kysymys käyttäjälle tulostettava kysymys
   * @param oletus arvo jota käytetään jos painetaan pelkkä ret
   * @return käyttäjän syöttämä merkkijono
   */
  public static String kysy(String kysymys, String oletus) {
    return kysy(kysymys,oletus,true);
  }

  /**
   * Aliohjelmalla kysytään käyttäjältä arvo merkkijonolle.
   * Ei tulosteta oletusarvoa.
   *
   * @param kysymys käyttäjälle tulostettava kysymys
   * @return käyttäjän syöttämä merkkijono
   */
  public static String kysy(String kysymys) {
    return kysy(kysymys,"",false);
  }

  /**
   * Kysytään reaaliluku. Jos annetaan ei-luku, kysytään uudelleen.
   * @param kysymys näytölle tulostettava kysymys
   * @param oletus arvo jota käytetään jos painetaanpelkkä Ret
   * @param tulOletus tulostetaanko oletus käyttäjälle
   * @return käyttäjän kirjoittama reaaliluku
   */
  public static double kysy(String kysymys, double oletus, boolean tulOletus) {
    while ( true ) {
      String s = kysy(kysymys,""+oletus,tulOletus);
      s = s.trim();
      if ( s.length() == 0 ) return oletus;
      try {
        return Double.parseDouble(s);
      } catch (NumberFormatException ex) {
        System.out.println("Ei numero: \"" + s + "\"");
      }
    }
  }

  /**
   * Kysytään reaaliluku. Jos annetaan ei-luku, kysytään uudelleen.
   * Oletusarvo tulostetaan sulkuihin.
   * @param kysymys näytölle tulostettava kysymys
   * @param oletus arvo jota käytetään jos painetaanpelkkä Ret
   * @return käyttäjän kirjoittama reaaliluku
   */
  public static double kysy(String kysymys, double oletus) {
    return kysy(kysymys,oletus,true);
  }

  /**
   * Kysytään reaaliluku. Jos annetaan ei-luku, kysytään uudelleen.
   * Ei tulosteta oletusta.
   * @param kysymys näytölle tulostettava kysymys
   * @return käyttäjän kirjoittama reaaliluku
   */
  public static double kysyDouble(String kysymys) {
    return kysy(kysymys,0.0,false);
  }

  /**
   * Kysytään kokonaisluku. Jos annetaan ei-luku, kysytään uudelleen.
   * @param kysymys näytölle tulostettava kysymys
   * @param oletus arvo jota käytetään jos painetaanpelkkä Ret
   * @param tulOletus tulostetaanko oletus käyttäjälle
   * @return käyttäjän kirjoittama kokonaisluku
   */
  public static int kysy(String kysymys, int oletus, boolean tulOletus) {
    while ( true ) {
      String s = kysy(kysymys,""+oletus,tulOletus);
      s = s.trim();
      if ( s.length() == 0 ) return oletus;
      try {
        return Integer.parseInt(s);
      } catch (NumberFormatException ex) {
        System.out.println("Ei kokonaisluku: \"" + s + "\"");
      }
    }
  }

  /**
   * Kysytään kokonaisluku. Jos annetaan ei-luku, kysytään uudelleen.
   * Oletusarvo tulostetaan sulkuihin.
   * @param kysymys näytölle tulostettava kysymys
   * @param oletus arvo jota käytetään jos painetaanpelkkä Ret
   * @return käyttäjän kirjoittama kokonaisluku
   */
  public static int kysy(String kysymys, int oletus) {
    return kysy(kysymys,oletus,true);
  }

  /**
   * Kysytään kokonaisluku. Jos annetaan ei-luku, kysytään uudelleen.
   * Ei tulosteta oletusta.
   * @param kysymys näytölle tulostettava kysymys
   * @return käyttäjän kirjoittama kokonaisluku
   */
  public static int kysyInt(String kysymys) {
    return kysy(kysymys,0,false);
  }


  
  /**
   * Testataan kysy-metodeja
   * @param args ei käytössä
   */
  public static void main(String[] args) {
    String s = kysy("Anna jono","Kissa");
    System.out.println("Jono oli: \"" + s + "\"");
    String s2 = kysy("Anna jono");
    System.out.println("Jono oli: \"" + s2 + "\"");
    double d = kysy("Anna reaaliluku",3.14);
    System.out.println("Luku oli: \"" + d + "\"");
    int i = kysyInt("Anna kokonaisluku");
    System.out.println("Luku oli: \"" + i + "\"");
    d = kysyDouble("Anna reaaliluku");
    System.out.println("Luku oli: \"" + d + "\"");
  }
}
